"""
Module for STAC Asset Generators
"""

from os import remove, listdir
from os.path import dirname, join, basename, abspath, basename

import pandas as pd
import rasterio
import pystac

from ...tools.metadata import remove_raster_metadata


MEDIA_TYPES_DICT = {
    "tif": pystac.MediaType.GEOTIFF,
    "tiff": pystac.MediaType.GEOTIFF,
    "png": pystac.MediaType.PNG,
    "jpg": pystac.MediaType.JPEG,
    "jpeg": pystac.MediaType.JPEG,
}


class STACAssetGenerator:
    """
    Standard STAC Asset Generator
    """

    type = "None"

    def __init__(self):
        pass

    @classmethod
    def extract_assets(cls, obj_info: pd.DataFrame):
        """
        Generate a single asset from the raster file

        :param raster_path: path to the raster file
        """
        # If there is no bands, create a single band asset from the file, assuming thats a singleband raster
        raster_path = obj_info["image"].values[0]
        title = basename(raster_path).split(".")[0]
        # Get the file extension
        raster_format = raster_path.split(".")[-1]
        asset = pystac.Asset(
            href=abspath(raster_path),
            title=title,
            media_type=MEDIA_TYPES_DICT[raster_format],
            roles=["data"],
        )

        return [asset]


class BandsAssetGenerator(STACAssetGenerator):
    """
    Bands STAC Asset Generator
    """

    type = "Bands"

    def __init__(self) -> None:
        super().__init__()

    def extract_assets(self, obj_info: pd.DataFrame):
        """
        Extract the assets from the raster file from the bands column

        :param raster_path: path to the raster file
        """
        asset_list = []
        # File path
        raster_path = obj_info["image"].values[0]
        # Bands
        bands = obj_info["bands"].values
        bands = bands[0] if bands else None

        if bands:
            with rasterio.open(raster_path, "r") as raster:
                raster_name = basename(raster_path).split(".")[0]
                if isinstance(bands, str):
                    bands = [bands]
                for band in bands:
                    i = bands.index(band)
                    raster_format = raster_path.split(".")[
                        -1
                    ]  # Will be used later to save the bands files
                    try:
                        single_band = raster.read(i + 1)
                    except IndexError:
                        single_band = raster.read(1)
                    band_name = f"{raster_name}_{band}.{raster_format}"
                    output_band = join(dirname(raster_path), band_name)
                    # Copy the metadata
                    metadata = raster.meta.copy()
                    metadata.update({"count": 1})
                    # Write the band to the output folder
                    with rasterio.open(output_band, "w", **metadata) as dest:
                        dest.write(single_band, 1)
                    # Instantiate pystac asset and append it to the list
                    asset_list.append(
                        pystac.Asset(
                            href=output_band,
                            title=band,
                            media_type=MEDIA_TYPES_DICT[raster_format],
                        )
                    )

            return asset_list



from pathlib import Path
import os
import json
import jwt


class AuthRepo:
    def __init__(self):
        self.algorithms = ["RS256"]
        self.base_path = str(Path.home()) + "/.cache/eotdl/"
        os.makedirs(self.base_path, exist_ok=True)
        self.creds_path = self.base_path + "creds.json"

    def save_creds(self, data):
        with open(self.creds_path, "w") as f:
            json.dump(data, f)
        return self.creds_path

    def load_creds(self):
        if os.path.exists(self.creds_path):
            with open(self.creds_path, "r") as f:
                creds = json.load(f)
            if not "id_token" in creds and not "api_key" in creds:
                return None
            if "api_key" in creds and creds["api_key"] != os.getenv(
                "EOTDL_API_KEY", None
            ):
                return None
            return creds
        return None

    def decode_token(self, token_data):
        return jwt.decode(
            token_data["id_token"],
            algorithms=self.algorithms,
            options={"verify_signature": False},
        )

    def logout(self):
        os.remove(self.creds_path)



import requests
import geopandas as gpd

from ..repos import APIRepo


class DatasetsAPIRepo(APIRepo):
    def __init__(self, url=None):
        super().__init__(url)

    def retrieve_datasets(self, name, limit):
        url = self.url + "datasets"
        if name is not None:
            url += "?match=" + name
        if limit is not None:
            if name is None:
                url += "?limit=" + str(limit)
            else:
                url += "&limit=" + str(limit)
        response = requests.get(url)
        return self.format_response(response)

    def create_dataset(self, metadata, user):
        response = requests.post(
            self.url + "datasets",
            json=metadata,
            headers=self.generate_headers(user),
        )
        return self.format_response(response)

    def retrieve_dataset(self, name):
        response = requests.get(self.url + "datasets?name=" + name)
        return self.format_response(response)

    def create_version(self, dataset_id, user):
        response = requests.post(
            self.url + "datasets/version/" + dataset_id,
            headers=self.generate_headers(user),
        )
        return self.format_response(response)

    def create_stac_dataset(self, name, user):
        response = requests.post(
            self.url + "datasets/stac",
            json={"name": name},
            headers=self.generate_headers(user),
        )
        return self.format_response(response)

    def ingest_stac(self, stac_json, dataset_id, user):
        response = requests.put(
            self.url + f"datasets/stac/{dataset_id}",
            json={"stac": stac_json},
            headers=self.generate_headers(user),
        )
        return self.format_response(response)

    def download_stac(self, dataset_id, user):
        url = self.url + "datasets/" + dataset_id + "/download"
        headers = self.generate_headers(user)
        response = requests.get(url, headers=headers)
        if response.status_code != 200:
            return None, response.json()["detail"]
        return gpd.GeoDataFrame.from_features(response.json()["features"]), None
